﻿using LibreriaMulticast;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

/*
 *         Progetto: Progetto "Trasmissioni multicast"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 13/06/2016
 *  
 *      Descrizione: Riceve messaggi multicast utilizzando le funzioni
 *                   di "LibreriaMulticast".
 *                   
 */

namespace RiceviMulticast
{
    class Program
    {
        static IPAddress RicercaIpHostLAN()
        {

            NetworkInterface[] interfaces = NetworkInterface.GetAllNetworkInterfaces();
            foreach (NetworkInterface adapter in interfaces)
            {
                IPInterfaceProperties prop = adapter.GetIPProperties();
                if (prop.GatewayAddresses.Count > 0)
                    foreach (var a in adapter.GetIPProperties().UnicastAddresses)
                    {
                        if (a.Address.AddressFamily == AddressFamily.InterNetwork)
                            return a.Address;
                    }
            }

            return null;
        }

        static void MainUdpClient(string[] args)
        {
            String[] indirizzi = new String[] {
                "224.5.5.5",
                "224.4.4.4"
            };
            const int porta = 2016;
            IPAddress ipLocale = RicercaIpHostLAN();
            IPEndPoint epLocale = new IPEndPoint(ipLocale, porta);

            Console.WriteLine("Listener avviato su {0}.", ipLocale);
            Console.WriteLine(" In attesa di messaggi sugli indirizzi multicast: ");

            foreach (string ip in indirizzi)
                Console.Write("{0}:{1}   ", ip, porta);
            Console.WriteLine();
            try
            {
                UdpClient listener = new UdpClient(epLocale);
                foreach (string ip in indirizzi)
                    listener.JoinMulticastGroup(IPAddress.Parse(ip));

                while (true)
                {
                    IPEndPoint epMittente = null;
                    byte[] bytes = listener.Receive(ref epMittente);
                    string msg = Encoding.UTF8.GetString(bytes, 0, bytes.Length);
                    Console.WriteLine(msg);
                }
                
            }
            catch (Exception ex)
            {
                Console.Error.WriteLine("Errore: {0}", ex.Message);
            }

        }

        static string RiceviMessaggioSocket(IPEndPoint epMulticast, IPAddress ipLocale)
        {
            string msg;
            byte[] bytes = new Byte[1000];
            EndPoint remoteEp = (EndPoint) new IPEndPoint(IPAddress.Any, 0);

            try
            {
                using (var sck = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp))
                {
                    MulticastOption opzione = new MulticastOption(epMulticast.Address, ipLocale);

                    sck.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.AddMembership, opzione);
                    sck.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.MulticastTimeToLive, 1);
                    

                    sck.Bind(new IPEndPoint(ipLocale, epMulticast.Port));


                    sck.ReceiveFrom(bytes, ref remoteEp);
                    msg = String.Format("Ricevuto da {0}: {1}", remoteEp, Encoding.UTF8.GetString(bytes, 0, bytes.Length));
                    

                    sck.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.DropMembership, opzione);
                }

            }
            catch (Exception ex)
            {
                msg = ex.Message;
            }

            return msg;
        }

        static String[] indirizzi = new String[] {
                "224.4.4.4",
                "224.5.5.5"
            };

        public static void Stampa(string msg, IPEndPoint epMittente, IPEndPoint epDestinatario)
        {
            int i;
            for (i = 0; i < indirizzi.Length; i++)
            {
                if (indirizzi[i].Equals(epDestinatario.Address.ToString()))
                    break;
            }

            Console.ForegroundColor = i % 2 == 0 ? ConsoleColor.Yellow : ConsoleColor.Red;
            Console.WriteLine("Messaggio da {0} a {1}", epMittente, epDestinatario);
            Console.ResetColor();
            Console.WriteLine(msg);
            Console.WriteLine();
        }

        static void Main(string[] args)
        {
            const int porta = 2016;
            IPAddress ipLocale = UtilityMulticast.RicercaIpHostLAN();
            ArrayList sck = new ArrayList();

            // Genera un socket e un thread per ogni indirizzo multicast

            for (int i = 0; i < indirizzi.Length; i++)
            {
                IPEndPoint epMulticast = new IPEndPoint(IPAddress.Parse(indirizzi[i]), porta);

                Socket s = UtilityMulticast.AvviaListenerUDP(
                    epMulticast,
                    ipLocale,
                    Stampa);

                sck.Add(s);

                Console.WriteLine("Avviato socket UDP {0}", epMulticast);
            }

            Console.WriteLine();
            Console.WriteLine("Premere INVIO per chiudere i listener");
            Console.WriteLine();

            Console.ReadLine();

            int isck = 0;
            foreach (Socket s in sck)
            {
                IPEndPoint epMulticast = new IPEndPoint(IPAddress.Parse(indirizzi[isck++]), porta);
                UtilityMulticast.ChiudiListenerUdp(
                    s,
                    epMulticast,
                ipLocale);
                Console.WriteLine("Chiuso socket {0}", epMulticast);
            }

            Console.WriteLine();
            Console.WriteLine("Premere INVIO per chiudere l'applicazione");
            Console.ReadLine();
        }

    }
}
